#[cfg(test)]
use super::*;

#[test]
fn clockwise() {
    let mut current = KartesianDirection::Top;
    let clock = [
        KartesianDirection::Top,
        KartesianDirection::TopRight,
        KartesianDirection::Right,
        KartesianDirection::BottomRight,
        KartesianDirection::Bottom,
        KartesianDirection::BottomLeft,
        KartesianDirection::Left,
        KartesianDirection::TopLeft,
    ];
    for hand in clock {
        assert_eq!(current, hand);
        current = current.clockwise(true);
    }
}
#[test]
fn anticlockwise() {
    let mut current = KartesianDirection::Top;
    let clock = [
        KartesianDirection::Top,
        KartesianDirection::TopLeft,
        KartesianDirection::Left,
        KartesianDirection::BottomLeft,
        KartesianDirection::Bottom,
        KartesianDirection::BottomRight,
        KartesianDirection::Right,
        KartesianDirection::TopRight,
    ];
    for hand in clock {
        assert_eq!(current, hand);
        current = current.anticlockwise(true);
    }
}

#[test]
fn kartesian_iter() {
    let test: Vec<_> = [
        KartesianDirection::Right,
        KartesianDirection::Top,
        KartesianDirection::Left,
        KartesianDirection::Bottom,
        KartesianDirection::None,
        KartesianDirection::TopLeft,
        KartesianDirection::TopRight,
        KartesianDirection::BottomLeft,
        KartesianDirection::BottomRight,
    ]
    .into();
    let i: Vec<_> = KartesianDirection::iter(true, true, true).collect();
    assert_eq!(i, test);
    let test: Vec<_> = [
        KartesianDirection::Right,
        KartesianDirection::Top,
        KartesianDirection::Left,
        KartesianDirection::Bottom,
        KartesianDirection::TopLeft,
        KartesianDirection::TopRight,
        KartesianDirection::BottomLeft,
        KartesianDirection::BottomRight,
    ]
    .into();
    let i: Vec<_> = KartesianDirection::iter(false, true, true).collect();
    assert_eq!(i, test);
}

#[test]
fn diff_dir() {
    const START: Kartesian<u8> = Kartesian::new(1, 1);
    // Top Group
    assert_eq!(KartesianDirection::TopLeft, START.diff(Kartesian::new(0, 0)).1);
    assert_eq!(KartesianDirection::Top, START.diff(Kartesian::new(0, 1)).1);
    assert_eq!(KartesianDirection::TopRight, START.diff(Kartesian::new(0, 2)).1);
    // Same Line
    assert_eq!(KartesianDirection::Left, START.diff(Kartesian::new(1, 0)).1);
    assert_eq!(KartesianDirection::None, START.diff(Kartesian::new(1, 1)).1);
    assert_eq!(KartesianDirection::Right, START.diff(Kartesian::new(1, 2)).1);
    // Below/Bottom Line
    assert_eq!(KartesianDirection::BottomLeft, START.diff(Kartesian::new(2, 0)).1);
    assert_eq!(KartesianDirection::Bottom, START.diff(Kartesian::new(2, 1)).1);
    assert_eq!(KartesianDirection::BottomRight, START.diff(Kartesian::new(2, 2)).1);
}

#[test]
fn moving_wrapping() {
    let max: Kartesian<u32> = Kartesian::new(10, 10);
    let mut pos = Kartesian::new(0, 0);
    let velocity = Velocity::new(1, 1, KartesianDirection::BottomRight);
    pos = pos.wrapping_move_velocity(velocity, max);
    assert_eq!(pos, Kartesian::new(1, 1));
    for _ in 1..10 {
        pos = pos.wrapping_move_velocity(velocity, max);
    }
    assert_eq!(pos, Kartesian::new(0, 0));
    let max = Kartesian::<u32>::new(7, 11);
    assert_eq!(Kartesian::new(2, 0).wrapping_move_velocity(Velocity::new(1, 3, KartesianDirection::TopLeft), max), Kartesian::new(1, 9));
    assert_eq!(Kartesian::new(4, 2).wrapping_move_velocity(Velocity::new(3, 2, KartesianDirection::TopRight), max), Kartesian::new(1, 4))
}

#[test]
fn test_wrap() {
    assert_eq!(wrap::<u32, 10>(11, 11), 0)
}
