use std::{
    ops::DivAssign,
    time::{Duration, Instant},
};

use num::*;

mod coordinate_systems;
pub mod strings;
pub use advent_of_code_macros::*;
pub use coordinate_systems::*;

pub type Table = Vec<Vec<char>>;

pub fn matrix(zero: bool, diag: bool, non_diag: bool) -> Vec<(i32, i32)> {
    const MIN: i32 = -1;
    const MAX: i32 = 1;
    let mut d = Vec::new();
    for i in MIN..=MAX {
        for j in MIN..=MAX {
            let ia = i.abs();
            let ja = j.abs();
            if zero && i == 0 && j == 0 {
                d.push((i, j));
            }
            if diag && ia == 1 && ja == 1 {
                d.push((i, j));
            }
            if non_diag && ia != ja && ia + ja == 1 {
                d.push((i, j));
            }
        }
    }
    d
}

pub fn numberlength_base<T: Integer + DivAssign + From<u32> + Copy, const BASE: u32>(n: T) -> u32 {
    let divider = T::from(BASE);
    let mut num = n;
    let mut length = 0;
    loop {
        if num < divider {
            return length + 1;
        }
        num /= divider;
        length += 1;
    }
}

#[inline]
pub fn numberlength<T: Integer + DivAssign + From<u32> + Copy>(n: T) -> u32 {
    numberlength_base::<T, 10>(n)
}

pub fn time_function<T, F: FnOnce() -> T>(func: F) -> (Duration, T) {
    let now = Instant::now();
    let o = func();
    (now.elapsed(), o)
}

#[derive(Debug, Clone, Copy, Default, PartialEq, Eq, PartialOrd, Ord)]
pub enum ExtendedOption<T> {
    #[default]
    Unset,
    Some(T),
    None,
}

/// exports an alias to an value of an enum.
#[macro_export]
macro_rules! enum_alias {
    ($enum:ident, $alias:ident, $target:ident) => {
            #[allow(non_upper_case_globals)]
            pub const $alias: $enum = <$enum>::$target;
    };
    ($visibility:vis $enum:ident, $alias:ident, $target:ident) => {
        #[allow(non_upper_case_globals)]
        $visibility const $alias: $enum = <$enum>::$target;
    };
}

/// reexports an Constant into an constant of an Trait
#[macro_export]
macro_rules! predefined_const {
    ($trait_name:ident, $const:ident, $t:ty) => {
        impl $trait_name for $t {
            const $const: $t = <$t>::$const;
        }
    };
}
