use num::{pow::Pow, Integer};
use std::ops::AddAssign;

#[inline]
pub fn char_to_num<T: Integer + From<u32>>(c: char) -> T {
    match c {
        '1' => T::one(),
        '2' => T::from(2),
        '3' => T::from(3),
        '4' => T::from(4),
        '5' => T::from(5),
        '6' => T::from(6),
        '7' => T::from(7),
        '8' => T::from(8),
        '9' => T::from(9),
        '0' => T::zero(),
        '-' => T::zero(),
        _ => unreachable!(),
    }
}

pub fn parsenumber<T: Integer + From<u32> + Pow<u32, Output = T> + AddAssign + Copy>(input: &str) -> T {
    const MAX_POWER: u32 = 32;
    let base = T::from(10);
    let mut output: T = T::from(0);
    let mut mul: u32 = 0;
    for c in input.chars().rev() {
        let i: T = char_to_num(c);
        if mul > MAX_POWER {
            return output;
        }
        output += i * base.pow(mul);
        mul += 1;
    }
    output
}

pub fn get_numbers<T: Integer + From<u32>>(input: &str) -> Vec<T> {
    let mut numbers = Vec::new();
    for char in input.chars() {
        numbers.push(char_to_num(char));
    }
    numbers
}

pub fn get_string_numbers(input: &str) -> Vec<u32> {
    let len = input.len();
    let mut output = Vec::new();
    for i in 0..len {
        match input.get(i..i + 1).unwrap() {
            "1" => {
                output.push(1);
                continue;
            },
            "2" => {
                output.push(2);
                continue;
            },
            "3" => {
                output.push(3);
                continue;
            },
            "4" => {
                output.push(4);
                continue;
            },
            "5" => {
                output.push(5);
                continue;
            },
            "6" => {
                output.push(6);
                continue;
            },
            "7" => {
                output.push(7);
                continue;
            },
            "8" => {
                output.push(8);
                continue;
            },
            "9" => {
                output.push(9);
                continue;
            },
            "0" => {
                output.push(0);
                continue;
            },
            _ => {},
        }
        if (i + 3) <= len {
            match input.get(i..i + 3).unwrap() {
                "one" => {
                    output.push(1);
                    continue;
                },
                "two" => {
                    output.push(2);
                    continue;
                },
                "six" => {
                    output.push(6);
                    continue;
                },
                _ => {},
            };
        }
        if (i + 4) <= len {
            match input.get(i..i + 4).unwrap() {
                "four" => {
                    output.push(4);
                    continue;
                },
                "five" => {
                    output.push(5);
                    continue;
                },
                "nine" => {
                    output.push(9);
                    continue;
                },
                "zero" => {
                    output.push(0);
                    continue;
                },
                _ => {},
            };
        }
        if (i + 5) <= len {
            match input.get(i..i + 5).unwrap() {
                "three" => {
                    output.push(3);
                    continue;
                },
                "seven" => {
                    output.push(7);
                    continue;
                },
                "eight" => {
                    output.push(8);
                    continue;
                },
                _ => {},
            };
        }
    }
    output
}

#[inline]
pub fn line_to_char(line: &str) -> Vec<char> {
    Vec::from_iter(line.chars())
}

pub fn convert_to_array<T, F: FnMut(&str) -> T, const S: char>(input: &str, func: F) -> Vec<T> {
    input.trim().split(S).map(str::trim_ascii).map(func).collect()
}
